#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAX(a,b) (a)>(b)?(a):(b)

int main(){
	int n, IterMax;
	int i, iter;
	double dx, Omega, Resid, dPhi, dPhiMax, Bf, Eps, Xmax, X, T;
	double *Phi, *Rhs;
	double *rAd, *Ar, *Al;
	FILE *fp;

/* INIT. */
	fp = fopen("cinput.dat", "r");
	fscanf(fp, "%d", &n);
	fscanf(fp, "%lf %lf", &dx, &Bf);
	fscanf(fp, "%d", &IterMax);
	fscanf(fp, "%lf %lf", &Eps, &Omega);
	fclose(fp);

	if(Omega <= 0.0){
		double pi = M_PI;
		Omega = 2.0/(1.0+sin(pi/(double)n));
	}

	printf("### OMEGA=%16.6e\n", Omega);

	Phi = calloc(n+1, sizeof(double));
	rAd = calloc(n, sizeof(double));
	Ar  = calloc(n, sizeof(double));
	Al  = calloc(n, sizeof(double));
	Rhs = calloc(n, sizeof(double));
	
	for(i=0;i<n;i++){
		Ar[i] = 1.0/dx;
		Al[i] = 1.0/dx;
		rAd[i] = 1.0/(-2.0/dx);
		Rhs[i] = -Bf*dx;
                Phi [i] = 0.0;
	}
	Rhs[n-1] = -Bf*dx;

	Al[0] = 0.0;
	Ar[0] = 0.0;
	rAd[0] = 1.0;
	Rhs[0] = 0.0;

	Ar[n-1] = 0.0;
	rAd[n-1] = 1.0/(-1.0/dx);
 
/*-- ITERATIONs */
	for(iter=0;iter<IterMax;iter++){
		dPhiMax = -1.0;
		for(i=0;i<n;i++){
			Resid = Rhs[i] - Al[i]*Phi[i-1] - Ar[i]*Phi[i+1];
			dPhi = Omega * (Resid*rAd[i]-Phi[i]);
			dPhiMax = MAX(fabs(dPhi), dPhiMax);
			Phi[i] += dPhi;
		}
         	  if((iter+1)%1000 == 0){
		  printf("%8d%s%16.6e%s%16.6e\n",
				  iter+1, " iters, RESID=", dPhiMax, " PHI(N)= ", Phi[n-1]);}
		if(dPhiMax<Eps) break;
	}

/*-- OUTPUT */
        printf("%8d%s%16.6e%s%16.6e\n",
	       iter+1, " iters, RESID=", dPhiMax, " PHI(N)= ", Phi[n-1]);

        Xmax= ((n-1)+0.5)*dx;
	for(i=0;i<n;i++){
          X= i*dx;
          T= -0.5*Bf*(X*X) + Bf*Xmax*X;
		printf("%8d%16.6E%16.6E\n", i+1, Phi[i], T);
	}

	return 0;
}
