!C
!C   1D Poisson Equation Solver by
!C   SOR (Successive Over Relaxation) Method 
!C
!C   d/dx(dPHI/dx) + BF = 0
!C   PHI=0@x=0  
!C
      program SOR_poi
      implicit REAL*8 (A-H,O-Z)

      integer :: N, ITERmax
      real(kind=8) :: dx, OMEGA, RESID, dPHI, dPHImax, BF, EPS 
      real(kind=8), dimension(:), allocatable :: PHI, RHS
      real(kind=8), dimension(:), allocatable :: rAD, AR, AL

!C
!C-- INIT.
      open  (11, file='input.dat', status='unknown')
       read (11,*) N
       read (11,*) dX, BF
       read (11,*) ITERmax
       read (11,*) EPS, OMEGA
      close (11)

      if (OMEGA.le.0.d0) then
        PI= 4.d0 * atan(1.d0)
        OMEGA= 2.d0/(1.d0+dsin(PI/dfloat(N)))
      endif

      write (*,'(a, 1pe16.6)') '### OMEGA=', OMEGA

      allocate (PHI(N+1), rAD(N), AR(N), AL(N), RHS(N))
       PHI= 0.d0
       AR = 1.d0/dX                 
       AL = 1.d0/dX                 
      rAD = 1.d0/(-2.d0/dX)
      RHS=  -BF * dX

       AL (1)= 0.d0
       AR (1)= 0.d0
      rAD (1)= 1.d0
       RHS(1)= 0.d0

       AR (N)= 0.d0
      rAD (N)= 1.d0/(-1.d0/dX)
 
!C
!C-- ITERATIONs
      do iter= 1, ITERmax
        dPHImax= -1.d0
        do i= 1, N
          RESID  = RHS(i) - AL(i)*PHI(i-1) - AR(i)*PHI(i+1)
          dPHI   = OMEGA * (RESID*rAD(i)-PHI(i))
          dPHImax= dmax1 (dabs(dPHI), dPHImax)
          PHI(i) = PHI(i) + dPHI
        enddo

        if (mod(iter,1000).eq.0) then
        write (*,'(i8, a, 1pe16.6, a, 1pe16.6)')                        &
     &         iter, ' iters, RESID=', dPHImax, ' PHI(N)= ', PHI(N)
        endif

        if (dPHImax.lt.EPS) exit
      enddo     

!C
!C-- OUTPUT
        write (*,'(i8, a, 1pe16.6, a, 1pe16.6)')                        &
     &         iter, ' iters, RESID=', dPHImax, ' PHI(N)= ', PHI(N)

      Xmax= (dfloat(N-1)+0.5d0)*dX
      do i= 1, N
        XX= dfloat(i-1)*dX
        T = -0.5d0*BF*(XX**2) + BF*Xmax*XX
        write (*,'(i8, 2(1pe16.6))') i, PHI(i), T
      enddo

      end program SOR_poi
